# Generated by Django 4.0.2 on 2022-02-25 21:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0007_briefurldefaultredirect'),
    ]

    operations = [
        migrations.AlterField(
            model_name='briefurldefaultredirect',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site', unique=True, verbose_name='site'),
        ),
    ]
