# Generated by Django 4.0.2 on 2022-02-24 22:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_burl', '0003_briefurl_site_alter_briefurl_burl_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='briefurl',
            name='burl_site_enabled',
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['burl', 'site', 'enabled'], name='django_burl_briefurl_brl_st_on'),
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['user', 'site'], name='django_burl_briefurl_user_site'),
        ),
    ]
