# Generated by Django 4.0.2 on 2022-02-25 17:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0004_remove_briefurl_burl_site_enabled_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='BriefURLDomainUser',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('role', models.CharField(choices=[('creator', 'Creator'), ('editor', 'Editor'), ('admin', 'Admin')], default='creator', max_length=16)),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
