# Generated by Django 2.2.27 on 2022-02-17 21:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_burl", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="briefurl",
            name="id",
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]
