# Generated by Django 4.0.2 on 2022-02-24 20:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0002_auto_20220217_1532'),
    ]

    operations = [
        migrations.AddField(
            model_name='briefurl',
            name='site',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='sites.site'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='briefurl',
            name='burl',
            field=models.CharField(blank=True, db_index=True, max_length=2048, verbose_name='Brief URL'),
        ),
        migrations.AlterField(
            model_name='briefurl',
            name='enabled',
            field=models.BooleanField(db_index=True, default=True),
        ),
        migrations.AlterUniqueTogether(
            name='briefurl',
            unique_together={('burl', 'site')},
        ),
        migrations.AddIndex(
            model_name='briefurl',
            index=models.Index(fields=['burl', 'site', 'enabled'], name='burl_site_enabled'),
        ),
    ]
