# Generated by Django 2.0.5 on 2019-02-21 22:47

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="BriefURL",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "url",
                    models.URLField(db_index=True, max_length=2048, verbose_name="URL"),
                ),
                (
                    "burl",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        max_length=2048,
                        unique=True,
                        verbose_name="Brief URL",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        blank=True,
                        help_text="description of the destination URL",
                        max_length=255,
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]
