# Generated by Django 4.0.2 on 2022-02-25 21:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('django_burl', '0006_alter_briefurldomainuser_unique_together'),
    ]

    operations = [
        migrations.CreateModel(
            name='BriefURLDefaultRedirect',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('url', models.URLField(verbose_name='redirect to')),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='site')),
            ],
        ),
    ]
